==> matter-bootstrap-1.3.1.tgz: package/out/collectionService.lua <==
-- Compiled with roblox-ts v2.3.0-dev-576ad98
local TS = _G[script]
local Object = TS.import(script, TS.getModule(script, "@rbxts", "object-utils"))
local decoratedComponentsIn = TS.import(script, script.Parent, "utils").decoratedComponentsIn
local TAG_METADATA_KEY = TS.import(script, script.Parent, "constants").TAG_METADATA_KEY
local Reflect = TS.import(script, TS.getModule(script, "@rbxts", "experimental-reflect").out).Reflect
local CollectionService = TS.import(script, TS.getModule(script, "@rbxts", "services")).CollectionService
local MatterCollectionService
do
	MatterCollectionService = setmetatable({}, {
		__tostring = function()
			return "MatterCollectionService"
		end,
	})
	MatterCollectionService.__index = MatterCollectionService
	function MatterCollectionService.new(...)
		local self = setmetatable({}, MatterCollectionService)
		return self:constructor(...) or self
	end
	function MatterCollectionService:constructor(world, components)
		self.world = world
		self.components = components
		self.tagToComponentCtorMap = {}
	end
	function MatterCollectionService:spawn(componentCtor, instance)
		local entity = self.world:spawn(componentCtor(), self.components.Model and self.components.Model({
			instance = instance,
		}))
		instance:SetAttribute("entityId", entity)
	end
	function MatterCollectionService:setupSpawning()
		for tag, componentCtor in self.tagToComponentCtorMap do
			for _, instance in Object.values(CollectionService:GetTagged(tag)) do
				self:spawn(componentCtor, instance)
			end
			CollectionService:GetInstanceAddedSignal(tag):Connect(function(instance)
				return self:spawn(componentCtor, instance)
			end)
			CollectionService:GetInstanceRemovedSignal(tag):Connect(function(instance)
				local entity = instance:GetAttribute("entityId")
				if not (entity ~= 0 and entity == entity and entity) then
					return warn(`No entityId attribute for {instance:GetFullName()}`)
				end
				self.world:despawn(entity)
			end)
		end
	end
	function MatterCollectionService:setupMap()
		for _, _binding in Object.entries(decoratedComponentsIn(self.components, TAG_METADATA_KEY)) do
			local componentKey = _binding[1]
			local componentCtor = _binding[2]
			local tag = Reflect.getMetadata(TAG_METADATA_KEY, self.components, componentKey)
			self.tagToComponentCtorMap[tag] = componentCtor
		end
	end
	function MatterCollectionService:setupCollectionService()
		self:setupMap()
		self:setupSpawning()
	end
end
return {
	default = MatterCollectionService,
}

==> matter-bootstrap-1.3.2.tgz: package/out/collectionService.lua <==
-- Compiled with roblox-ts v2.3.0-dev-576ad98
local TS = _G[script]
local Object = TS.import(script, TS.getModule(script, "@rbxts", "object-utils"))
local decoratedComponentsIn = TS.import(script, script.Parent, "utils").decoratedComponentsIn
local TAG_METADATA_KEY = TS.import(script, script.Parent, "constants").TAG_METADATA_KEY
local Reflect = TS.import(script, TS.getModule(script, "@rbxts", "experimental-reflect").out).Reflect
local CollectionService = TS.import(script, TS.getModule(script, "@rbxts", "services")).CollectionService
local MatterCollectionService
do
	MatterCollectionService = setmetatable({}, {
		__tostring = function()
			return "MatterCollectionService"
		end,
	})
	MatterCollectionService.__index = MatterCollectionService
	function MatterCollectionService.new(...)
		local self = setmetatable({}, MatterCollectionService)
		return self:constructor(...) or self
	end
	function MatterCollectionService:constructor(world, components)
		self.world = world
		self.components = components
		self.tagToComponentCtorMap = {}
	end
	function MatterCollectionService:spawn(componentCtor, instance)
		local entity = if self.components.Model then self.world:spawn(componentCtor(), self.components.Model()) else self.world:spawn(componentCtor())
		instance:SetAttribute("entityId", entity)
	end
	function MatterCollectionService:setupSpawning()
		for tag, componentCtor in self.tagToComponentCtorMap do
			for _, instance in Object.values(CollectionService:GetTagged(tag)) do
				self:spawn(componentCtor, instance)
			end
			CollectionService:GetInstanceAddedSignal(tag):Connect(function(instance)
				return self:spawn(componentCtor, instance)
			end)
			CollectionService:GetInstanceRemovedSignal(tag):Connect(function(instance)
				local entity = instance:GetAttribute("entityId")
				if not (entity ~= 0 and entity == entity and entity) then
					return warn(`No entityId attribute for {instance:GetFullName()}`)
				end
				self.world:despawn(entity)
			end)
		end
	end
	function MatterCollectionService:setupMap()
		for _, _binding in Object.entries(decoratedComponentsIn(self.components, TAG_METADATA_KEY)) do
			local componentKey = _binding[1]
			local componentCtor = _binding[2]
			local tag = Reflect.getMetadata(TAG_METADATA_KEY, self.components, componentKey)
			self.tagToComponentCtorMap[tag] = componentCtor
		end
	end
	function MatterCollectionService:setupCollectionService()
		self:setupMap()
		self:setupSpawning()
	end
end
return {
	default = MatterCollectionService,
}
